# This program reads the breathing and whisking data

using SmoothingSplines
using Statistics
using Formatting
using Printf

include("read_bw_types.jl")

# This function reads the data.
function read_data(data_store, runpar, fl)

    i_previous_number = zeros(Int64, 2)
    i_line = 0
#    store_one_record = Array{Array{Float64,1},1}(undef, 2)
#    [store_one_record[ilr] = Array{Float64,1}(undef, 0) for ilr in 1:2]
    
#   l=1 r=2
    
    while !eof(fl.dat)
        line_read = readline(fl.dat)
        line_split = split(line_read, ",")
	i_line += 1
	tt = parse(Float64, line_split[4])
	bb = parse(Float64, line_split[3])
	
	for ilr in 1:2
	    if  line_split[ilr] != "NaN"
	        ww = parse(Float64, line_split[ilr])
		record = [tt, bb, ww, i_line]
#		println("ilr=", ilr, " record=", record, " aw=", line_split[ilr],
#		  " ", i_previous_number[ilr], " len=", length(data_store[ilr]))
		
		if i_previous_number[ilr] == 0
		    push!(data_store[ilr], [record])
		else
		    push!(data_store[ilr][end], record)
		end
		
		i_previous_number[ilr] += 1
	    else
	        i_previous_number[ilr] = 0
	    end
        end
    end

end #read_data

# This function analyzes the data.
function analyze_data(data_store, runpar, fl)

    for ilr in 1:2
        for iepisode in 1:length(data_store[ilr])
	   len_episode = length(data_store[ilr][iepisode])
	   diff_t_episode = iepisode > 1 ? data_store[ilr][iepisode][1][1] -
	     data_store[ilr][iepisode-1][end][1] : 0.0
	   len_t_episode = data_store[ilr][iepisode][end][1] -
	     data_store[ilr][iepisode][1][1]
	   printfmtln(fl.out, "{1:d} {2:f} {3:d} {4:d} {5:f} {6:d} {7:f} " *
	     "{8:f} {9:f}", iepisode, diff_t_episode, len_episode, ilr,
	     data_store[ilr][iepisode][1][1],
	     round(Int64, data_store[ilr][iepisode][1][4]), len_t_episode,
	     data_store[ilr][iepisode][end][1], data_store[ilr][iepisode][1][1])
	end
	println(fl.out, "  ")
    end

end #analyze_data

# This function unites episodes with time difference <= nfill*deltat between them
function unite_data(data_store, runpar, fl)

    for ilr in 1:2
	iepisode = 1
	while iepisode < length(data_store[ilr])
	    diff_iepisode = round(Int64, data_store[ilr][iepisode+1][1][4] -
	      data_store[ilr][iepisode][end][4])
            if diff_iepisode <= runpar.nfill
	        println("ilr=", ilr, " i_ep=", iepisode)
	        additional_episode = Array{Array{Float64,1},1}(undef,0)
		record_beg = data_store[ilr][iepisode][end]
		record_end = data_store[ilr][iepisode+1][1]
		for i_add in 1:diff_iepisode
		    tt = record_beg[1] + (1.0 * i_add / diff_iepisode) *
		      (record_end[1] - record_beg[1])
		    bb = record_beg[2] + (1.0 * i_add / diff_iepisode) *
		      (record_end[2] - record_beg[2])
	  	    ww = record_beg[3] + (1.0 * i_add / diff_iepisode) *
		      (record_end[3] - record_beg[3])
		    i_line = record_beg[4] + i_add
		    push!(additional_episode, [tt, bb, ww, i_line])
		end
		append!(append!(data_store[ilr][iepisode], additional_episode),
		    data_store[ilr][iepisode+1])
		deleteat!(data_store[ilr], iepisode+1)
	    else
	        iepisode += 1
	    end
	end
    end

end #unite_data

# This function writes the data for the left and right whiskers.
function write_data(data_store, runpar, fl)

    for ilr in 1:2
        for iepisode in 1:length(data_store[ilr])	      
            iepisode > 1 ? println(fl.colar[ilr], " ") : nothing
            for record in data_store[ilr][iepisode]
	        printfmtln(fl.colar[ilr], "{1:f} {2:f} {3:f} {4:f}", record[1],
		  record[2], record[3], record[4])
	    end
	end
    end

end #write_data

# This function normalizes the breathing data
function normalize_breath_whisk(data_store_ilr_iepisode, runpar, fl)

    normanized_data_ar = Array{Array{Float64,1},1}(undef, 0)
    av_val = zeros(2)
    sd_val = zeros(2)

    for ibw in 1:2 #breathing, whisking
        ar_col = map(x -> x[ibw+1], data_store_ilr_iepisode)
        av_val[ibw] = Statistics.mean(ar_col)
        sd_val[ibw] = Statistics.std(ar_col)
        printfmtln("ibw={1:d} av={2:e} sd={3:e}", ibw, av_val[ibw], sd_val[ibw])
 	if sd_val[ibw] < runpar.epsilon
 	    printfmtln("ibw={2:d} av={3:f} sd={4:f} < epsilon={5:f}", ibw,
	      av_val[ibw], sd_val[ibw], epsilon)
 	    exit(0)
    	end
    end

    factor_s = sd_val[2] / sd_val[1]
    println("xxx")
    irecord = 0
    for record in data_store_ilr_iepisode
        irecord += 1
        tt = irecord * runpar.deltat
        b_val = av_val[2] + factor_s * (record[2] - av_val[1])
	push!(normanized_data_ar, [tt, b_val, record[3], record[4]])
    end

#    for record in data_store_ilr_iepisode
#        push!(normanized_data_ar, record)
#    end
    
    return normanized_data_ar
end #normalize_breath_whisk

# This function writes the norlamized breathing and whisking data for
# each episode separately.
function write_normalize_data(data_store, runpar, fl)

   for ilr in 1:2
       long_episode_num = 0
       
       for iepisode in 1:length(data_store[ilr])
	    len_t_episode = data_store[ilr][iepisode][end][1] -
	      data_store[ilr][iepisode][1][1]
	      
	    if len_t_episode > runpar.len_t_min
	        long_episode_num += 1

                normanized_data_ar = normalize_breath_whisk(
		  data_store[ilr][iepisode], runpar, fl)
                
		file_write_name = "tbw1/$ilr.$long_episode_num"
		println("file_write_name=", file_write_name)
		fl.bbb=open(file_write_name * ".b", "w")
		fl.www=open(file_write_name * ".w", "w")

                time_record = 0.0
                for record in normanized_data_ar
		    time_record += runpar.deltat = 0.0041
		    printfmtln(fl.bbb, "{1:f} {2:f}", time_record,
		      record[2])
		    printfmtln(fl.www, "{1:f} {2:f}", time_record,
		      record[3])
#		    printfmtln(fl.bbb, "{1:f} {2:f} {3:f}", record[1],
#		      record[2], time_record)
#		    printfmtln(fl.www, "{1:f} {2:f} {3:f}", record[1],
#		      record[3], time_record)
		end
		
		close(fl.bbb)
		close(fl.www)
	    end
	end
	
	println("ilr=", ilr, " long_episode_num=", long_episode_num)
    end

end #write_normalize_data

# This function processes the data from a file.
function process_data(runpar, fl)

#   [lr n_episode i_in_episode t,s,w]
    data_store = Array{Array{Array{Array{Float64,1},1},1},1}(undef, 2) #LR
    [data_store[ilr] = Array{Array{Array{Float64,1},1},1}(undef, 0) for ilr
      in 1:2]

    read_data(data_store, runpar, fl)
#   analyze_data(data_store, runpar, fl)
    unite_data(data_store, runpar, fl)
    write_data(data_store, runpar, fl)
    analyze_data(data_store, runpar, fl)

    dir_name = "tbw1"
    if !isdir(dir_name)
        mkdir(dir_name)
    end    
    write_normalize_data(data_store, runpar, fl)
    
end #process_data

#main

fl = fl_st()
fl.dat = open("Breathing_WhiskingL_WhiskingR", "r")
fl.colar = Array{IOStream,1}(undef, 2)
[fl.colar[ilr] = open("bw.col." * "$ilr", "w") for ilr in 1:2]
fl.out = open("bw.out", "w")

runpar = run_par()
runpar.epsilon = 1.0e-10
runpar.nfill = 4
runpar.deltat = 0.0041
runpar.len_t_min = 1.0

process_data(runpar, fl)

close(fl.dat)
[close(fl.colar[ilr]) for ilr in 1:2]
close(fl.out)
