# This program reads a csv file

using DelimitedFiles
using Formatting
using Printf
using Statistics
using DSP

# This function writes the data columns of one trace: time, whisking, sniffing
# on files. The sniffing signal is normalized such that its SD will be the equal
# to the SD of the whisking signal.
function write_data(file_num, data_table)

    ws_array = ["w", "s"]  #whisking, sniffing
    num_columns = length(data_table[1, :])
    num_traces = div(num_columns, 3)
    num_t = length(data_table[:, 1])

    av_val = zeros(num_traces, 2)
    sd_val = zeros(num_traces, 2)

    fcol_mat = Array{IOStream, 2}(undef, num_traces, 2)

    dir_name = "tws_files_" * string(file_num)
    if !isdir(dir_name)
        mkdir(dir_name)
    end
    
    for itrace in 1:num_traces, iws in 1:2
        file_name = dir_name * "/tws." * string(itrace) * "." *
	  string(ws_array[iws])
	println("file_name=", file_name)
        fcol_mat[itrace, iws] =  open(file_name, "w")
    end

    for itrace in 1:num_traces
        for iws in 1:2
            col_x_num = iws * num_traces + itrace
            av_val[itrace, iws]  = Statistics.mean(data_table[:, col_x_num])
            sd_val[itrace, iws]  = Statistics.std(data_table[:, col_x_num])
#	    printfmtln("itrace={1:d} iws={2:d} av={3:e} sd={4:e}", itrace, iws,
#	      av_val[itrace, iws], sd_val[itrace, iws])
	    if sd_val[itrace, iws] < epsilon
	        printfmtln("itrace={1:d} iws={2:d} av={3:f} sd={4:f} " *
		"< epsilon={5:f}", itrace, iws, av_val[itrace, iws],
		sd_val[itrace, iws], epsilon)
		exit(0)
	    end
	end
	
	printfmt(fout, "itrace={1:2d}", itrace)
	printfmt(fout, " w: av={1:9.5f} sd={2:9.5f}", av_val[itrace, 1],
	  sd_val[itrace, 1])
	printfmtln(fout, " s: av={1:12.5e} sd={2:12.5e}", av_val[itrace, 2],
	  sd_val[itrace, 2])
    end
    
    for itrace in 1:num_traces, iws in 1:2
	col_x_num = iws * num_traces + itrace
#	println("itr=", itrace, " iws=", iws, " col=", col_x_num)

#       ratio = iws == 1 ? 1 :  sd_val[itrace, 1] /  sd_val[itrace, 2]

       for it in 1:num_t
            y_val = iws == 1 ? data_table[it, col_x_num] : av_val[itrace, 1] +
	      (data_table[it, col_x_num] - av_val[itrace, 2]) *
	      sd_val[itrace, 1] /  sd_val[itrace, 2]
            println(fcol_mat[itrace, iws], data_table[it, itrace], " ", -y_val)
	end
    end
    
    for itrace in 1:num_traces, iws in 1:2
        close(fcol_mat[itrace, iws])
    end
    
#   for itrace in 1:num_traces
#        for iws in 1:2
#	    printfmtln("{1:2d} {2:10.5f} {3:10.5f} {4:e} {5:e}", itrace,
#	      av_val[itrace, 1], sd_val[itrace, 1],
#	      av_val[itrace, 2], sd_val[itrace, 2])	    
#	end
#   end

end #function write_data

# main

epsilon = 1.0e-10

data_file_names = [
"swdata_RS2_05242010_sniff_angle.mat.csv" # old 1
"swdata_RS2_05252010_sniff_angle.mat.csv"
"swdata_RS2_05272010_sniff_angle.mat.csv"
"swdata_RS2_06042010_sniff_angle.mat.csv"
"swdata_RS2_06132010_sniff_angle.mat.csv"
"swdata_RS2_06172010_sniff_angle.mat.csv"
"swdata_RS2_06252010_sniff_angle.mat.csv"
"swdata_RS2_06272010_sniff_angle.mat.csv"
"swdata_RS2_06302010_sniff_angle.mat.csv"
"swdata_RS3_06252010_sniff_angle.mat.csv"
"swdata_RS3_06262010_sniff_angle.mat.csv"
"swdata_RS3_06272010_sniff_angle.mat.csv"
"swdata_RS3_06302010_sniff_angle.mat.csv"
"swdata_RS3_07012010_sniff_angle.mat.csv"
"swdata_RS4_06252010_sniff_angle.mat.csv"
"swdata_RS4_07012010_sniff_angle.mat.csv" # old 16
"swdata_RS5_08052010_sniff_airpuff_angle.mat.csv" #new 17
"swdata_RS5_08052010_sniff_ammpuff_angle.mat.csv"
"swdata_RS5_08062010_sniff_airpuff_angle.mat.csv"
"swdata_RS5_08062010_sniff_ammpuff_angle.mat.csv"
"swdata_RS5_08062010_sniff_angle.mat.csv" # 21
"swdata_RS5_08092010_sniff_airpuff_angle.mat.csv"
"swdata_RS5_08092010_sniff_ammpuff_angle.mat.csv"
"swdata_RS5_08092010_sniff_angle.mat.csv" # 24
"swdata_RS6_08202010_sniff2_angle.mat.csv" # 25
"swdata_RS6_08202010_sniff_angle.mat.csv" # 26
"swdata_RS6_08222010_sniff2_angle.mat.csv" # 27
"swdata_RS6_08222010_sniff_angle.mat.csv" # 28
"swdata_RS6_08242010_sniff_airpuff_angle.mat.csv"
"swdata_RS6_08242010_sniff_ammpuff_angle.mat.csv"
"swdata_RS6_08242010_sniff_angle.mat.csv" #31
"swdata_RS7_09142010_sniff_angle.mat.csv" #32
"swdata_RS7_09222010_sniff_angle.mat.csv" #33
"swdata_RS9_11042010_sniff_airpuff_angle.mat.csv"
"swdata_RS9_11042010_sniff_ammpuff_angle.mat.csv" #new
]

fout = open("read_csv.out", "w")

for file_num in 1:length(data_file_names)
    data_file_str = "../" * data_file_names[file_num]
    data_table = readdlm(data_file_str, ',') #readcsv(data_file_str)

    write_data(file_num, data_table)
end

close(fout)
