# This program reads a csv file

using DelimitedFiles
using Formatting
using Printf
using Statistics
using DSP

#Linear interpolation 
function lininter(x1, x2, xc, y1, y2)

    linter = ((xc-x1)*y2+(x2-xc)*y1) / (x2-x1)
    return linter
end  #lininter

# This function finds the dropouts and gets rid of them using interpolations.
function get_rid_of_dropouts(file_num, data_table, data_nd)

    num_t = length(data_table[:, 1])
    drop_thr = 8.5
    drop_ar = Array{Array{Array{Int64,1},1},1}(undef, 2) #[irl][begin-end][it]
    for irl in 1:2
        drop_ar[irl] = Array{Array{Int64,1}}(undef, 2)
	[drop_ar[irl][ibe] = Array{Int64,1}(undef, 0) for ibe in 1:2]
    end
    
    
#   check whether there are dropouts at the edges.
    for irl in 1:2, i_begin_end in [1, num_t]
       if data_table[i_begin_end, irl] > drop_thr
           println("irl=", irl, " i_begin_end=", i_begin_end, " val=",
             data_table[i_begin_end, irl])
       end
    end

#   find intervals of dropouts.
    for irl in 1:2 #wr wl
        for it in 2:num_t-1
#	    println(fout, irl, " ", it, " ", data_table[it-1, irl+1], " ",
#             data_table[it, irl+1])
            if -data_table[it-1, irl+1] < drop_thr &&
	      -data_table[it, irl+1] > drop_thr
	        push!(drop_ar[irl][1], it)
	    elseif -data_table[it, irl+1] > drop_thr &&
	      -data_table[it+1, irl+1] < drop_thr
	        push!(drop_ar[irl][2], it)
	    end
	end
	if length(drop_ar[irl][1]) != length(drop_ar[irl][2])
	    println("length(drop_ar[", irl, "][1]=", length(drop_ar[irl][1]),
	      " != length(drop_ar[", irl, "][2]=", length(drop_ar[irl][2]))
	      exit(0)
	end
	println(fout, "irl=", irl, " lenb=", length(drop_ar[irl][1]), " lene=",
	  length(drop_ar[irl][2]))
	for id in 1:length(drop_ar[irl][1])
	    t1 = data_table[drop_ar[irl][1][id]-1, 1]
	    t2 = data_table[drop_ar[irl][2][id]+1, 1]
	    printfmtln(fout, "{1:d} {2:d} {3:f} {4:f} {5:f}",
	      drop_ar[irl][1][id], drop_ar[irl][2][id], t2-t1, t1, t2)
	end
    end


    data_nd[:,:] = data_table[:,:]
#   interpolate over intervals of dropouts.

    for irl in 1:2 #wr wl
        nd = 0
        for it in 1:num_t
	   if nd < length(drop_ar[irl][1])
	       if it == drop_ar[irl][1][nd+1]
	           nd += 1
	       end
	   end

           if nd > 0
	       if it <= drop_ar[irl][2][nd]
	           it1 = drop_ar[irl][1][nd]-1
	           it2 = drop_ar[irl][2][nd]+1
	           yy = lininter(data_table[it1, 1], data_table[it2, 1],
	             data_table[it, 1], data_table[it1, irl+1],
		     data_table[it2, irl+1])
                   data_nd[it, irl+1] = yy
		   println(fout, "irl=", irl, " nd=", nd, " it=", it, " yy=", yy)
	      end
	  end

	end
	println(fout, "irl=", irl, " nd=", nd)
    end
   
end

# This function writes the data columns of one trace: time, whisking, sniffing
# on files. The sniffing signal is normalized such that its SD will be the equal
# to the SD of the whisking signal.
function write_data(file_num, data_table, ratio_tws_double)

#   whisking left, whisking right, sniffing.
    ws_array = ["wr", "wl", "s"]

    num_columns = length(data_table[1, :])
    if num_columns != 4
        println("num_columns=", num_columns, "!=4")
	exit(0)
    end
    num_t = length(data_table[:, 1])
    println("num_t=", num_t)

    av_val = zeros(3)
    sd_val = zeros(3)

    fcol_mat = Array{IOStream}(undef, 4)

    dir_name = "double_" * string(file_num)
    if !isdir(dir_name)
        mkdir(dir_name)
    end
    
    for iws in 1:2
        file_name = dir_name * "/" * ws_array[iws]
        println("file_name=", file_name)
        fcol_mat[2*iws-1] =  open(file_name, "w")
        file_name = dir_name * "/" * "s" * ws_array[iws][2] 
        println("file_name=", file_name)
        fcol_mat[2*iws] =  open(file_name, "w")
    end
    
    for iws in 1:3
        col_x_num = iws + 1
        av_val[iws]  = Statistics.mean(data_table[:, col_x_num])
        sd_val[iws]  = Statistics.std(data_table[:, col_x_num])
#	printfmtln("iws={1:d} av={2:e} sd={3:e}", iws, av_val[iws], sd_val[iws])
 	if sd_val[iws] < epsilon
 	    printfmtln("iws={2:d} av={3:f} sd={4:f} < epsilon={5:f}", iws,
	      av_val[iws], sd_val[iws], epsilon)
 	     exit(0)
 	end
 	printfmt(fout, " {1:s}: av={2:9.5f} sd={3:9.5f}", ws_array[iws],
	  av_val[iws], sd_val[iws])
    end
    printfmt(fout, "\n")
    av_lr = mean(av_val)
    sd_lr = mean(sd_val)

    s_index = 4
    for iws in 1:2 #"wr", "wl"
        col_x_num = iws + 1
	factor_s = sd_val[iws] /  sd_val[s_index-1]
	
	for it in 1:num_t
#           w
            Vval = data_table[it, col_x_num]
	    w_angle = atan(Vval - set_point[iws][2], 0.0 - set_point[iws][1]) *
	      180.0 / pi
#	    w_angle = Vval - set_point[iws][2]
	    println(fcol_mat[2*iws-1], data_table[it, 1], " ",
	      w_angle, " ", Vval, " ", set_point[iws][2])
#	      -data_table[it, col_x_num] * ratio_tws_double)
#	    s
	    y_val = av_val[iws] + factor_s * data_table[it, s_index]
	    println(fcol_mat[2*iws], data_table[it, 1], " ",
	      -y_val * ratio_tws_double)
	end
    end
    
    for iws in 1:4
        close(fcol_mat[iws])
    end
    
end #function write_data

# main

epsilon = 1.0e-10
ratio_tws_double = 1 #32.54
set_point = [[-0.68, 0.0], [-0.68, -0.5]] #[[-0.68, -0.88], [-0.68, -0.35]]

data_file_names = [
"doubleWhisk_20130405_RS164_a.mat.csv"
"doubleWhisk_20130405_RS164_a2.mat.csv"
]

fout = open("read_double.out", "w")

for file_num in 1:length(data_file_names)
    data_file_str = "../" * data_file_names[file_num]
    data_table = readdlm(data_file_str, ',') #readcsv(data_file_str)

    data_nd = zeros(length(data_table[:, 1]), 4) #t, wr, wl
    get_rid_of_dropouts(file_num, data_table, data_nd)
    
    write_data(file_num, data_nd, ratio_tws_double) # data_table)
end

close(fout)
